@@ -1,6 +1,8 @@
-{-# OPTIONS_GHC -XMultiParamTypeClasses -XScopedTypeVariables -XFlexibleContexts -XRankNTypes
-    -XExistentialQuantification #-}
-
+{-# LANGUAGE FlexibleContexts #-}
+{-# LANGUAGE MultiParamTypeClasses #-}
+{-# LANGUAGE ExistentialQuantification #-}
+{-# LANGUAGE RankNTypes #-}
+{-# LANGUAGE OverloadedStrings #-}
 -----------------------------------------------------------------------------
 --
 -- Module      :  Graphics.UI.Editor.Basics
@@ -23,7 +25,6 @@ module Graphics.UI.Editor.Basics (
 ,   Extractor
 ,   Applicator
 ,   Editor
-,   getStandardRegFunction
 ,   emptyNotifier
 ,   GUIEvent(..)
 ,   GUIEventSelector(..)
@@ -40,20 +41,30 @@ module Graphics.UI.Editor.Basics (
 ,   propagateAsChanged
 ) where
 
+import Prelude
+import Text.Show
+
 import Graphics.UI.Gtk
-import qualified Graphics.UI.Gtk.Gdk.Events as Gtk
 import Data.Unique
 import Data.IORef
+import Data.Text (Text)
 import Control.Monad
+import Control.Monad.Trans (liftIO)
 
 import Graphics.UI.Editor.Parameters
 import Control.Event
 import Data.Map (Map(..))
 import qualified Data.Map as Map  (delete,insert,lookup,empty)
-import Data.Maybe (isJust,fromJust)
+import Data.Maybe (fromMaybe, isJust, fromJust)
 import Unsafe.Coerce (unsafeCoerce)
 import Control.Arrow (first)
 import MyMissing (allOf)
+import qualified Data.Text as T (pack)
+
+fromString = Just . T.pack
+
+ifThenElse True t _ = t
+ifThenElse _ _ f = f
 
 -- ---------------------------------------------------------------------
 -- * Basic Types
@@ -93,8 +104,7 @@ type Editor alpha  =   Parameters -> Notifier
 --
 data GUIEvent = GUIEvent {
     selector :: GUIEventSelector
-,   gtkEvent :: Gtk.Event
-,   eventText :: String
+,   eventText :: Text
 ,   gtkReturn :: Bool -- ^ True means that the event has been completely handled,
                       --  gtk shoudn't do any further action about it (Often not
                       --  a good idea
@@ -124,12 +134,12 @@ genericGUIEvents = [FocusOut,FocusIn,ButtonPressed,KeyPressed]
 --  | A type for handling an IO event
 --  Returning True: The event has been handles
 --  Returning False: Handling should proceed
-type GtkHandler = Gtk.Event -> IO Bool
+type GtkHandler = IO Bool
 
 --
 -- | A type for a function to register a gtk event
 -- |
-type GtkRegFunc = forall o . GObjectClass o => o -> GtkHandler -> IO (Connection)
+type GtkRegFunc = forall o . GObjectClass o => o -> GtkHandler -> IO Connection
 
 --
 -- | The widgets are the real event sources.
@@ -260,31 +270,28 @@ propagateEvent (Noti pairRef) eventSources eventSel = do
 --
 
 activateEvent
-  :: (GObjectClass o) =>
+  :: GObjectClass o =>
      o
      -> Notifier
-     -> Maybe (o -> GtkHandler -> IO Connection)
+     -> Maybe (o -> IO Bool -> IO Connection)
      -> GUIEventSelector
      -> IO ()
 activateEvent widget (Noti pairRef) mbRegisterFunc eventSel = do
-    let registerFunc    =   case mbRegisterFunc of
-                                Just f  ->  f
-                                Nothing ->  getStandardRegFunction eventSel
-    cid <- registerFunc widget (\ e -> do
+    let registerFunc    =   fromMaybe (getStandardRegFunction eventSel) mbRegisterFunc
+    cid <- registerFunc widget (do
                 (hi,_) <- readIORef pairRef
                 case Map.lookup eventSel hi of
                     Nothing -> return False
                     Just [] -> return False
                     Just handlers -> do
-                        name <- if (widget `isA` gTypeWidget)
+                        name <- if widget `isA` gTypeWidget
                                     then widgetGetName (castToWidget widget)
-                                    else return "no widget - no name"
-                        eventList <- mapM (\f -> do
-                            let ev = GUIEvent eventSel e "" False
-                            f ev)
-                                (map snd handlers)
+                                    else return "no widget - no name" :: IO Text
+                        eventList <- mapM ((\f -> do
+                            let ev = GUIEvent eventSel "" False
+                            f ev) . snd) handlers
                         let boolList = map gtkReturn eventList
-                        return (foldr (&&) True boolList))
+                        return (and boolList))
     (handerls,ger) <- readIORef pairRef
     let newGer      =   case Map.lookup eventSel ger of
                             Nothing ->  Map.insert eventSel ([cid],([],Map.empty))
@@ -297,12 +304,11 @@ activateEvent widget (Noti pairRef) mbRegisterFunc eventSel = do
 -- | A convinence method for not repeating this over and over again
 --
 getStandardRegFunction :: GUIEventSelector -> GtkRegFunc
-getStandardRegFunction FocusOut         =   \w h -> liftM ConnectC $ (castToWidget w) `onFocusOut` h
-getStandardRegFunction FocusIn          =   \w h -> liftM ConnectC $ (castToWidget w) `onFocusIn` h
-getStandardRegFunction ButtonPressed    =   \w h -> liftM ConnectC $ (castToWidget w) `afterButtonRelease` h
-getStandardRegFunction KeyPressed       =   \w h -> liftM ConnectC $ (castToWidget w) `afterKeyRelease` h
-getStandardRegFunction Clicked          =   \w h -> liftM ConnectC $ (castToButton w) `onClicked`
-                                                                    (h (Gtk.Event True) >> return ())
+getStandardRegFunction FocusOut         =   \w h -> liftM ConnectC $ on (castToWidget w) focusOutEvent $ liftIO h
+getStandardRegFunction FocusIn          =   \w h -> liftM ConnectC $ on (castToWidget w) focusInEvent $ liftIO h
+getStandardRegFunction ButtonPressed    =   \w h -> liftM ConnectC $ after (castToWidget w) buttonReleaseEvent $ liftIO h
+getStandardRegFunction KeyPressed       =   \w h -> liftM ConnectC $ after (castToWidget w) keyReleaseEvent $ liftIO h
+getStandardRegFunction Clicked          =   \w h -> liftM ConnectC $ on (castToButton w) buttonActivated $ void $ liftIO h
 getStandardRegFunction _    =   error "Basic>>getStandardRegFunction: no original GUI event"
 
 registerEvents :: EventSource alpha beta gamma delta => alpha -> [delta] -> (beta -> gamma beta) -> gamma [Maybe Unique]
@@ -312,6 +318,6 @@ registerEvents notifier selectors handler =
 propagateAsChanged
   :: (EventSource alpha GUIEvent m GUIEventSelector) =>
      alpha -> [GUIEventSelector] -> m ()
-propagateAsChanged notifier selectors =
+propagateAsChanged notifier =
     mapM_ (\s -> registerEvent notifier s
-            (\ e -> triggerEvent notifier e{selector = MayHaveChanged})) selectors
+            (\ e -> triggerEvent notifier e{selector = MayHaveChanged}))
