@@ -1,3 +1,5 @@
+{-# LANGUAGE CPP #-}
+{-# LANGUAGE OverloadedStrings #-}
 -----------------------------------------------------------------------------
 --
 -- Module      :  Graphics.UI.Editor.Composite
@@ -23,7 +25,7 @@ module Graphics.UI.Editor.Composite (
 ,   ColumnDescr(..)
 
 ,   filesEditor
-,   stringsEditor
+,   textsEditor
 
 ,   versionEditor
 ,   versionRangeEditor
@@ -35,6 +37,8 @@ import Graphics.UI.Gtk
 import Control.Monad
 import Data.IORef
 import Data.Maybe
+import Data.Text (Text)
+import qualified Data.Text as T (pack, unpack, null)
 
 import Default
 import Control.Event
@@ -198,7 +202,7 @@ splitEditor (fstEd,fstPara) (sndEd,sndPara) parameters notifier = do
 -- | An editor with a subeditor which gets active, when a checkbox is selected
 -- or deselected (if the positive Argument is False)
 --
-maybeEditor :: Default beta => (Editor beta, Parameters) -> Bool -> String -> Editor (Maybe beta)
+maybeEditor :: Default beta => (Editor beta, Parameters) -> Bool -> Text -> Editor (Maybe beta)
 maybeEditor (childEdit, childParams) positive boolLabel parameters notifier = do
     coreRef      <- newIORef Nothing
     childRef     <- newIORef Nothing
@@ -240,7 +244,7 @@ maybeEditor (childEdit, childParams) positive boolLabel parameters notifier = do
                                 then do
                                     (childWidget,_,_) <- getChildEditor childRef childEdit childParams cNoti
                                     inj1 (not positive)
-                                    widgetHideAll childWidget
+                                    widgetHide childWidget
                                 else inj1 (not positive)
                         Just val ->
                             if hasChild
@@ -286,7 +290,7 @@ maybeEditor (childEdit, childParams) positive boolLabel parameters notifier = do
                                 hasChild <- hasChildEditor childRef
                                 when hasChild $ do
                                     (childWidget,_,_) <- getChildEditor childRef childEdit childParams cNoti
-                                    widgetHideAll childWidget
+                                    widgetHide childWidget
                             else do
                                 hasChild <- hasChildEditor childRef
                                 (childWidget,inj2,ext2) <- getChildEditor childRef childEdit childParams cNoti
@@ -316,7 +320,7 @@ maybeEditor (childEdit, childParams) positive boolLabel parameters notifier = do
 -- | An editor with a subeditor which gets active, when a checkbox is selected
 -- or grayed out (if the positive Argument is False)
 --
-disableEditor :: Default beta => (Editor beta, Parameters) -> Bool -> String -> Editor (Bool,beta)
+disableEditor :: Default beta => (Editor beta, Parameters) -> Bool -> Text -> Editor (Bool,beta)
 disableEditor (childEdit, childParams) positive boolLabel parameters notifier = do
     coreRef      <- newIORef Nothing
     childRef     <- newIORef Nothing
@@ -450,7 +454,7 @@ disableEditor (childEdit, childParams) positive boolLabel parameters notifier =
 -- | An editor with a subeditor which gets active, when a checkbox is selected
 -- or deselected (if the positive Argument is False)
 eitherOrEditor :: (Default alpha, Default beta) => (Editor alpha, Parameters) ->
-                        (Editor beta, Parameters) -> String -> Editor (Either alpha beta)
+                        (Editor beta, Parameters) -> Text -> Editor (Either alpha beta)
 eitherOrEditor (leftEditor,leftParams) (rightEditor,rightParams)
             label2 parameters notifier = do
     coreRef <- newIORef Nothing
@@ -546,7 +550,7 @@ eitherOrEditor (leftEditor,leftParams) (rightEditor,rightParams)
 -- and a nontrivial:
 --  [("Package",\(Dependency str _) -> [cellText := str])
 --  ,("Version",\(Dependency _ vers) -> [cellText := showVersionRange vers])])
-data ColumnDescr row = ColumnDescr Bool [(String,(row -> [AttrOp CellRendererText]))]
+data ColumnDescr row = ColumnDescr Bool [(Text,(row -> [AttrOp CellRendererText]))]
 
 --
 -- | An editor with a subeditor, of which a list of items can be selected
@@ -576,20 +580,20 @@ multisetEditor (ColumnDescr showHeaders columnsDD) (singleEditor, sParams) mbSor
                             return (castToBox b,castToButtonBox bb)
                     (frameS,injS,extS) <- singleEditor sParams cnoti
                     mapM_ (propagateEvent notifier [cnoti]) allGUIEvents
-                    addButton   <- buttonNewWithLabel "Add"
-                    removeButton <- buttonNewWithLabel "Remove"
+                    addButton   <- buttonNewWithLabel ("Add" :: Text)
+                    removeButton <- buttonNewWithLabel ("Remove" :: Text)
                     containerAdd buttonBox addButton
                     containerAdd buttonBox removeButton
                     listStore   <-  listStoreNew ([]:: [alpha])
                     activateEvent listStore notifier
                         (Just (\ w h -> do
                             res     <-  after (castToTreeModel w) rowInserted (\ _ _ ->
-                                h (Gtk.Event True) >> return ())
+                                h >> return ())
                             return (ConnectC res))) MayHaveChanged
                     activateEvent listStore notifier
                         (Just (\ w h -> do
                             res     <-  after (castToTreeModel w) rowDeleted (\ _ ->
-                                h (Gtk.Event True) >> return ())
+                                h >> return ())
                             return (ConnectC res))) MayHaveChanged
                     treeView        <-  treeViewNewWithModel listStore
                     let minSize =   getParameter paraMinSize parameters
@@ -597,6 +601,9 @@ multisetEditor (ColumnDescr showHeaders columnsDD) (singleEditor, sParams) mbSor
                     sw          <-  scrolledWindowNew Nothing Nothing
                     containerAdd sw treeView
                     scrolledWindowSetPolicy sw PolicyAutomatic PolicyAutomatic
+#ifdef MIN_VERSION_gtk3
+                    scrolledWindowSetMinContentHeight sw (snd minSize)
+#endif
                     sel         <-  treeViewGetSelection treeView
                     treeSelectionSetMode sel SelectionSingle
                     mapM_ (\(str,func) -> do
@@ -609,7 +616,7 @@ multisetEditor (ColumnDescr showHeaders columnsDD) (singleEditor, sParams) mbSor
                             cellLayoutSetAttributes col renderer listStore func
                         ) columnsDD
                     treeViewSetHeadersVisible treeView showHeaders
-                    sel  `onSelectionChanged` selectionHandler sel listStore injS
+                    on sel treeSelectionSelectionChanged $ selectionHandler sel listStore injS
                     boxPackStart box sw PackGrow 0
                     boxPackStart box buttonBox PackNatural 0
                     boxPackStart box frameS PackNatural 0
@@ -620,7 +627,7 @@ multisetEditor (ColumnDescr showHeaders columnsDD) (singleEditor, sParams) mbSor
                         (case mbSort of
                             Nothing -> vs
                             Just sortF -> sortF vs)
-                    addButton `onClicked` do
+                    on addButton buttonActivated $ do
                         mbv <- extS
                         case mbv of
                             Just v -> do
@@ -647,10 +654,10 @@ multisetEditor (ColumnDescr showHeaders columnsDD) (singleEditor, sParams) mbSor
                                         mbCol <- treeViewGetColumn treeView 0
                                         case mbCol of
                                             Nothing  -> return ()
-                                            Just col -> treeViewScrollToCell treeView [idx] col Nothing
+                                            Just col -> treeViewScrollToCell treeView (Just [idx]) Nothing Nothing
                                     Nothing -> return ()
                             Nothing -> return ()
-                    removeButton `onClicked` do
+                    on removeButton buttonActivated $ do
                         mbi <- treeSelectionGetSelected sel
                         case mbi of
                             Nothing -> return ()
@@ -686,21 +693,21 @@ multisetEditor (ColumnDescr showHeaders columnsDD) (singleEditor, sParams) mbSor
                 return ()
 
 
-filesEditor :: Maybe FilePath -> FileChooserAction -> String -> Editor [FilePath]
+filesEditor :: Maybe FilePath -> FileChooserAction -> Text -> Editor [FilePath]
 filesEditor fp act label p =
     multisetEditor
-        (ColumnDescr False [("",(\row -> [cellText := row]))])
+        (ColumnDescr False [("",(\row -> [cellText := T.pack row]))])
         (fileEditor fp act label, emptyParams)
         (Just sort)
         (Just (==))
         (paraShadow <<<- ParaShadow ShadowIn $
             paraDirection  <<<- ParaDirection Vertical $ p)
 
-stringsEditor :: (String -> Bool) -> Bool -> Editor [String]
-stringsEditor validation trimBlanks p =
+textsEditor :: (Text -> Bool) -> Bool -> Editor [Text]
+textsEditor validation trimBlanks p =
     multisetEditor
         (ColumnDescr False [("",(\row -> [cellText := row]))])
-        (stringEditor validation trimBlanks, emptyParams)
+        (textEditor validation trimBlanks, emptyParams)
         (Just sort)
         (Just (==))
         (paraShadow <<<- ParaShadow ShadowIn $ p)
@@ -708,31 +715,25 @@ stringsEditor validation trimBlanks p =
 dependencyEditor :: [PackageIdentifier] -> Editor Dependency
 dependencyEditor packages para noti = do
     (wid,inj,ext) <- pairEditor
-        ((eitherOrEditor (comboSelectionEditor ((sort . nub) (map (display . pkgName) packages)) id
+        (comboEntryEditor ((sort . nub) (map (T.pack . display . pkgName) packages))
             , paraName <<<- ParaName "Select" $ emptyParams)
-            (stringEditor (const True) True, paraName <<<- ParaName "Enter" $ emptyParams)
-            "Select from list?"), paraName <<<- ParaName "Name"$ emptyParams)
         (versionRangeEditor,paraName <<<- ParaName "Version" $ emptyParams)
         (paraDirection <<<- ParaDirection Vertical $ para)
         noti
-    let pinj (Dependency pn@(PackageName s) v) = if elem s (map (display . pkgName) packages)
-                                                    then inj (Left s,v)
-                                                    else inj (Right s,v)
+    let pinj (Dependency pn@(PackageName s) v) = inj (T.pack s,v)
     let pext = do
         mbp <- ext
         case mbp of
             Nothing -> return Nothing
-            Just (Left "",v) -> return Nothing
-            Just (Left s,v) -> return (Just $ Dependency (PackageName s) v)
-            Just (Right "",v) -> return Nothing
-            Just (Right s,v) -> return (Just $ Dependency (PackageName s) v)
+            Just ("",v) -> return Nothing
+            Just (s,v) -> return (Just $ Dependency (PackageName (T.unpack s)) v)
     return (wid,pinj,pext)
 
 dependenciesEditor :: [PackageIdentifier] -> Editor [Dependency]
 dependenciesEditor packages p noti =
     multisetEditor
-        (ColumnDescr True [("Package",\(Dependency (PackageName str) _) -> [cellText := str])
-                           ,("Version",\(Dependency _ vers) -> [cellText := display vers])])
+        (ColumnDescr True [("Package",\(Dependency (PackageName str) _) -> [cellText := T.pack str])
+                           ,("Version",\(Dependency _ vers) -> [cellText := T.pack $ display vers])])
         (dependencyEditor packages,
             paraOuterAlignment <<<- ParaInnerAlignment (0.0, 0.5, 1.0, 1.0)
                 $ paraInnerAlignment <<<- ParaOuterAlignment (0.0, 0.5, 1.0, 1.0)
@@ -753,7 +754,7 @@ versionRangeEditor para noti = do
         maybeEditor
             ((eitherOrEditor
                 (pairEditor
-                    (comboSelectionEditor v1 show, emptyParams)
+                    (comboSelectionEditor v1 (T.pack . show), emptyParams)
                     (versionEditor, paraName <<<- ParaName "Enter Version" $ emptyParams),
                         (paraDirection <<<- ParaDirection Vertical
                             $ paraName <<<- ParaName "Simple"
@@ -763,7 +764,7 @@ versionRangeEditor para noti = do
                             $ paraInnerPadding <<<- ParaInnerPadding   (0, 0, 0, 0)
                             $ emptyParams))
                 (tupel3Editor
-                    (comboSelectionEditor v2 show, emptyParams)
+                    (comboSelectionEditor v2 (T.pack . show), emptyParams)
                     (versionRangeEditor, paraShadow <<<- ParaShadow ShadowIn $ emptyParams)
                     (versionRangeEditor, paraShadow <<<- ParaShadow ShadowIn $ emptyParams),
                         paraName <<<- ParaName "Complex"
@@ -777,6 +778,9 @@ versionRangeEditor para noti = do
             (paraDirection <<<- ParaDirection Vertical $ para)
             noti
     let vrinj AnyVersion                =   inj Nothing
+#if MIN_VERSION_Cabal(1,8,0)
+        vrinj (WildcardVersion v)       =   inj (Just (Left (WildcardVersionS,v)))
+#endif
         vrinj (ThisVersion v)           =   inj (Just (Left (ThisVersionS,v)))
         vrinj (LaterVersion v)          =   inj (Just (Left (LaterVersionS,v)))
         vrinj (EarlierVersion v)        =   inj (Just (Left (EarlierVersionS,v)))
@@ -796,6 +800,9 @@ versionRangeEditor para noti = do
                         Nothing -> return (Just AnyVersion)
                         Just Nothing -> return (Just AnyVersion)
                         Just (Just (Left (ThisVersionS,v)))     -> return (Just (ThisVersion v))
+#if MIN_VERSION_Cabal(1,8,0)
+                        Just (Just (Left (WildcardVersionS,v)))     -> return (Just (WildcardVersion v))
+#endif
                         Just (Just (Left (LaterVersionS,v)))    -> return (Just (LaterVersion v))
                         Just (Just (Left (EarlierVersionS,v)))   -> return (Just (EarlierVersion v))
 
@@ -807,13 +814,18 @@ versionRangeEditor para noti = do
                                                         -> return (Just (IntersectVersionRanges v1 v2))
     return (wid,vrinj,vrext)
         where
+#if MIN_VERSION_Cabal(1,8,0)
+            v1 = [ThisVersionS,WildcardVersionS,LaterVersionS,ThisOrLaterVersionS,EarlierVersionS,ThisOrEarlierVersionS]
+#else
             v1 = [ThisVersionS,LaterVersionS,ThisOrLaterVersionS,EarlierVersionS,ThisOrEarlierVersionS]
+#endif
             v2 = [UnionVersionRangesS,IntersectVersionRangesS]
 
-data Version1 = ThisVersionS | LaterVersionS | ThisOrLaterVersionS | EarlierVersionS | ThisOrEarlierVersionS
+data Version1 = ThisVersionS | WildcardVersionS | LaterVersionS | ThisOrLaterVersionS | EarlierVersionS | ThisOrEarlierVersionS
     deriving (Eq)
 instance Show Version1 where
     show ThisVersionS   =  "This Version"
+    show WildcardVersionS   =  "Wildcard Version"
     show LaterVersionS  =  "Later Version"
     show ThisOrLaterVersionS = "This or later Version"
     show EarlierVersionS =  "Earlier Version"
@@ -827,7 +839,7 @@ instance Show Version2 where
 
 versionEditor :: Editor Version
 versionEditor para noti = do
-    (wid,inj,ext) <- stringEditor (\s -> not (null s)) True para noti
+    (wid,inj,ext) <- stringEditor (not . null) True para noti
     let pinj v = inj (display v)
     let pext = do
         s <- ext
