@@ -1,4 +1,6 @@
-{-# OPTIONS_GHC -XScopedTypeVariables #-}
+{-# LANGUAGE CPP #-}
+{-# LANGUAGE ScopedTypeVariables #-}
+{-# LANGUAGE OverloadedStrings #-}
 -----------------------------------------------------------------------------
 --
 -- Module      :  Graphics.UI.Editor.Simple
@@ -19,6 +21,7 @@ module Graphics.UI.Editor.Simple (
 ,   boolEditor2
 ,   enumEditor
 ,   clickEditor
+,   textEditor
 ,   stringEditor
 ,   multilineStringEditor
 ,   intEditor
@@ -26,6 +29,7 @@ module Graphics.UI.Editor.Simple (
 ,   fontEditor
 ,   colorEditor
 ,   comboSelectionEditor
+,   comboEntryEditor
 ,   staticListEditor
 ,   staticListMultiEditor
 ,   multiselectionEditor
@@ -36,12 +40,7 @@ module Graphics.UI.Editor.Simple (
 ,   okCancelFields
 ) where
 
-#if MIN_VERSION_gtk(0,10,5)
-import Graphics.UI.Gtk hiding (eventKeyName, eventModifier)
-#else
 import Graphics.UI.Gtk
-#endif
-import qualified Graphics.UI.Gtk as Gtk
 import Control.Monad
 import Data.IORef
 import Data.List
@@ -52,17 +51,18 @@ import Graphics.UI.Editor.Parameters
 --import Graphics.UI.Editor.Basics
 import Graphics.UI.Editor.MakeEditor
 import Control.Event
-#if MIN_VERSION_gtk(0,10,5)
-import Graphics.UI.Gtk.Gdk.Events (Event(..))
-#else
-import Graphics.UI.Gtk.Gdk.Events (Event(..))
-#endif
-import MyMissing (trim, allOf)
+import MyMissing (allOf)
 import qualified Graphics.UI.Gtk.Gdk.Events as Gtk (Event(..))
 import Unsafe.Coerce (unsafeCoerce)
 import Graphics.UI.Editor.Basics
        (GUIEvent(..), GUIEventSelector(..), propagateAsChanged,
         genericGUIEvents, activateEvent, Editor)
+import Control.Exception as E (catch, IOException)
+import Control.Monad.IO.Class (MonadIO(..))
+import Control.Applicative ((<$>))
+import qualified Data.Text as T (strip, unpack, pack, empty)
+import Data.Monoid ((<>))
+import Data.Text (Text)
 
 -- ------------------------------------------------------------
 -- * Simple Editors
@@ -109,13 +109,13 @@ boolEditor parameters notifier = do
                 Just button -> do
                     r <- toggleButtonGetActive button
                     return (Just r))
-        (paraName <<<- ParaName "" $ parameters)
+        (paraName <<<- ParaName T.empty $ parameters)
         notifier
 
 --
 -- | Editor for a boolean value in the form of two radio buttons
 ----
-boolEditor2 :: String -> Editor Bool
+boolEditor2 :: Text -> Editor Bool
 boolEditor2 label2 parameters notifier = do
     coreRef <- newIORef Nothing
     mkEditor
@@ -128,8 +128,8 @@ boolEditor2 label2 parameters notifier = do
                     radio2 <- radioButtonNewWithLabelFromWidget radio1 label2
                     boxPackStart box radio1 PackGrow 2
                     boxPackStart box radio2 PackGrow 2
-                    widgetSetName radio1 $ getParameter paraName parameters ++ ".1"
-                    widgetSetName radio2 $ getParameter paraName parameters ++ ".2"
+                    widgetSetName radio1 $ getParameter paraName parameters <> ".1"
+                    widgetSetName radio2 $ getParameter paraName parameters <> ".2"
                     containerAdd widget box
                     if bool
                         then toggleButtonSetActive radio1 True
@@ -154,7 +154,7 @@ boolEditor2 label2 parameters notifier = do
 --
 -- | Editor for an enum value in the form of n radio buttons
 ----
-enumEditor :: forall alpha . (Show alpha, Enum alpha, Bounded alpha)  => [String] -> Editor alpha
+enumEditor :: forall alpha . (Show alpha, Enum alpha, Bounded alpha)  => [Text] -> Editor alpha
 enumEditor labels parameters notifier = do
     coreRef <- newIORef Nothing
     let vals :: [alpha] =  allOf
@@ -164,16 +164,16 @@ enumEditor labels parameters notifier = do
             case core of
                 Nothing  -> do
                     box <- vBoxNew True 2
-                    let label0 = if length labels > 0 then labels !! 0 else show (vals !! 0)
+                    let label0 = if length labels > 0 then labels !! 0 else T.pack (show (vals !! 0))
                     button0 <- radioButtonNewWithLabel label0
                     buttons <- mapM (\ v -> do
                         let n = fromEnum v
-                        let label = if length labels > n then labels !! n else show v
+                        let label = if length labels > n then labels !! n else T.pack (show v)
                         radio <- if n == 0
                                     then return button0
                                     else radioButtonNewWithLabelFromWidget button0 label
                         boxPackStart box radio PackGrow 2
-                        widgetSetName radio (label ++ show n)
+                        widgetSetName radio (label <> T.pack (show n))
                         return radio) vals
                     containerAdd widget box
                     mapM_
@@ -249,10 +249,10 @@ imageEditor parameters notifier = do
         notifier
 
 --
--- | Editor for a string in the form of a text entry
+-- | Editor for a Text in the form of a text entry
 --
-stringEditor :: (String -> Bool) -> Bool -> Editor String
-stringEditor validation trimBlanks parameters notifier = do
+textEditor :: (Text -> Bool) -> Bool -> Editor Text
+textEditor validation trimBlanks parameters notifier = do
     coreRef <- newIORef Nothing
     mkEditor
         (\widget string -> do
@@ -264,24 +264,32 @@ stringEditor validation trimBlanks parameters notifier = do
                     mapM_ (activateEvent (castToWidget entry) notifier Nothing) genericGUIEvents
                     propagateAsChanged notifier [KeyPressed]
                     containerAdd widget entry
-                    entrySetText entry (if trimBlanks then trim string else string)
+                    entrySetText entry (if trimBlanks then T.strip string else string)
                     writeIORef coreRef (Just entry)
-                Just entry -> entrySetText entry (if trimBlanks then trim string else string))
+                Just entry -> entrySetText entry (if trimBlanks then T.strip string else string))
         (do core <- readIORef coreRef
             case core of
                 Nothing -> return Nothing
                 Just entry -> do
                     r <- entryGetText entry
                     if validation r
-                        then return (Just (if trimBlanks then trim r else r))
+                        then return (Just (if trimBlanks then T.strip r else r))
                         else return Nothing)
         parameters
         notifier
 
+--
+-- | Editor for a String in the form of a text entry
+--
+stringEditor :: (String -> Bool) -> Bool -> Editor String
+stringEditor validation trimBlanks parameters notifier = do
+    (wid,inj,ext) <- textEditor (validation . T.unpack) True parameters notifier
+    return (wid, inj . T.pack, (T.unpack <$>) <$> ext)
+
 --
 -- | Editor for a multiline string in the form of a multiline text entry
 --
-multilineStringEditor :: Editor String
+multilineStringEditor :: Editor Text
 multilineStringEditor parameters notifier = do
     coreRef <- newIORef Nothing
     mkEditor
@@ -331,9 +339,7 @@ intEditor (min, max, step) parameters notifier = do
                     mapM_ (activateEvent (castToWidget spin) notifier Nothing) (genericGUIEvents)
                     activateEvent (castToWidget spin) notifier
                         (Just (\ w h -> do
-                            res     <-  afterValueSpinned (castToSpinButton w) (do
-                                h (Gtk.Event True)
-                                return ())
+                            res     <-  afterValueSpinned (castToSpinButton w) (h >> return ())
                             return (unsafeCoerce res))) MayHaveChanged
                     containerAdd widget spin
                     spinButtonSetValue spin (fromIntegral v)
@@ -359,8 +365,8 @@ genericEditor parameters notifier = do
         s <- ext
         case s of
             Nothing -> return Nothing
-            Just s -> catch (liftM Just (readIO s))
-                            (\e -> do
+            Just s -> E.catch (liftM Just (readIO s))
+                            (\(e :: IOException) -> do
                                 putStrLn ("Generic editor no parse for " ++ s ++ " " ++ show e)
                                 return Nothing)
     return (wid,ginj,gext)
@@ -389,8 +395,7 @@ buttonEditor parameters notifier = do
 --
 -- | Editor for the selection of some element from a static list of elements in the
 -- | form of a combo box
-
-comboSelectionEditor :: Eq beta => [beta] -> (beta -> String) -> Editor beta
+comboSelectionEditor :: Eq beta => [beta] -> (beta -> Text) -> Editor beta
 comboSelectionEditor list showF parameters notifier = do
     coreRef <- newIORef Nothing
     mkEditor
@@ -399,14 +404,13 @@ comboSelectionEditor list showF parameters notifier = do
             case core of
                 Nothing  -> do
                     combo <- comboBoxNewText
+                    widgetSetSizeRequest combo 200 (-1)
                     mapM_ (\o -> comboBoxAppendText combo (showF o)) list
                     widgetSetName combo (getParameter paraName parameters)
                     mapM_ (activateEvent (castToWidget combo) notifier Nothing) genericGUIEvents
                     activateEvent (castToWidget combo) notifier
                         (Just (\ w h -> do
-                            res     <-  on (castToComboBox w) changed (do
-                                h (Gtk.Event True)
-                                return ())
+                            res     <-  on (castToComboBox w) changed (h >> return ())
                             return (unsafeCoerce res))) MayHaveChanged
                     comboBoxSetActive combo 1
                     containerAdd widget combo
@@ -431,6 +435,52 @@ comboSelectionEditor list showF parameters notifier = do
         parameters
         notifier
 
+-- | Like comboSelectionEditor but allows entry of text not in the list
+comboEntryEditor :: [Text] -> Editor Text
+comboEntryEditor list parameters notifier = do
+    coreRef <- newIORef Nothing
+    mkEditor
+        (\widget obj -> do
+            core <- readIORef coreRef
+            case core of
+                Nothing  -> do
+                    combo <- comboBoxNewWithEntry
+                    comboBoxSetModelText combo
+                    widgetSetSizeRequest combo 200 (-1)
+                    mapM_ (comboBoxAppendText combo) list
+                    widgetSetName combo (getParameter paraName parameters)
+                    mapM_ (activateEvent (castToWidget combo) notifier Nothing) genericGUIEvents
+                    activateEvent (castToWidget combo) notifier
+                        (Just (\ w h -> do
+                            res     <-  on (castToComboBox w) changed (h >> return ())
+                            return (unsafeCoerce res))) MayHaveChanged
+                    comboBoxSetActive combo 1
+                    containerAdd widget combo
+                    let ind = elemIndex obj list
+                    case ind of
+                        Just i -> comboBoxSetActive combo i
+                        Nothing -> do
+                            mbEntry <- binGetChild combo
+                            case mbEntry of
+                                Nothing -> return ()
+                                Just entry -> entrySetText (castToEntry entry) obj
+                    writeIORef coreRef (Just combo)
+                Just combo -> do
+                    let ind = elemIndex obj list
+                    case ind of
+                        Just i -> comboBoxSetActive combo i
+                        Nothing -> return ())
+        (do core <- readIORef coreRef
+            case core of
+                Nothing -> return Nothing
+                Just combo -> do
+                    mbEntry <- binGetChild combo
+                    case mbEntry of
+                        Nothing -> return Nothing
+                        Just entry -> Just <$> entryGetText (castToEntry entry))
+        parameters
+        notifier
+
 --
 -- | Editor for the selection of some elements from a list of elements in the
 -- | form of a list box
@@ -454,7 +504,7 @@ multiselectionEditor parameters notifier = do
                     treeViewAppendColumn listView col
                     cellLayoutPackStart col renderer True
                     cellLayoutSetAttributes col renderer listStore
-                        $ \row -> [ cellText := show row ]
+                        $ \row -> [ cellText := T.pack (show row) ]
                     treeViewSetHeadersVisible listView False
                     listStoreClear listStore
                     mapM_ (listStoreAppend listStore) objs
@@ -481,7 +531,7 @@ multiselectionEditor parameters notifier = do
 -- | Editor for the selection of some elements from a static list of elements in the
 -- | form of a list box with toggle elements
 
-staticListMultiEditor :: (Eq beta) => [beta] -> (beta -> String) -> Editor [beta]
+staticListMultiEditor :: (Eq beta) => [beta] -> (beta -> Text) -> Editor [beta]
 staticListMultiEditor list showF parameters notifier = do
     coreRef <- newIORef Nothing
     mkEditor
@@ -518,22 +568,26 @@ staticListMultiEditor list showF parameters notifier = do
                     containerAdd sw listView
                     scrolledWindowSetPolicy sw PolicyAutomatic PolicyAutomatic
                     containerAdd widget sw
+#ifdef MIN_VERSION_gtk3
+                    scrolledWindowSetMinContentHeight sw (snd minSize)
+#endif
                       -- update the model when the toggle buttons are activated
                     on rendererToggle cellToggled $ \pathStr -> do
                         let (i:_) = stringToTreePath pathStr
                         val <- listStoreGetValue listStore i
                         listStoreSetValue listStore i (not (fst val),snd val)
-                    listView `onKeyPress` (\event -> do
-                        let Key { eventKeyName = name, eventModifier = modifier, eventKeyChar = char } = event
-                        case (name, modifier, char) of
-                            ("Return", _, _) -> do
-                                sel <- treeViewGetSelection listView
-                                rows <- treeSelectionGetSelectedRows sel
-                                mapM_ (\ (i:_) -> do
-                                    val <- listStoreGetValue listStore i
-                                    listStoreSetValue listStore i (not (fst val),snd val)) rows
-                                return True
-                            _ -> return False)
+                    listView `on` keyPressEvent $ do
+                        name <- eventKeyName
+                        liftIO $ do
+                            case name of
+                                "Return" -> do
+                                    sel <- treeViewGetSelection listView
+                                    rows <- treeSelectionGetSelectedRows sel
+                                    mapM_ (\ (i:_) -> do
+                                        val <- listStoreGetValue listStore i
+                                        listStoreSetValue listStore i (not (fst val),snd val)) rows
+                                    return True
+                                _ -> return False
                     writeIORef coreRef (Just (listView,listStore))
                 Just (listView,listStore) -> do
                     let model = map (\e -> (elem e objs,e)) list
@@ -552,7 +606,7 @@ staticListMultiEditor list showF parameters notifier = do
 -- | Editor for the selection of some elements from a static list of elements in the
 -- | form of a list box
 
-staticListEditor :: (Eq beta) => [beta] -> (beta -> String) -> Editor beta
+staticListEditor :: (Eq beta) => [beta] -> (beta -> Text) -> Editor beta
 staticListEditor list showF parameters notifier = do
     coreRef <- newIORef Nothing
     mkEditor
@@ -585,6 +639,9 @@ staticListEditor list showF parameters notifier = do
                     containerAdd sw listView
                     scrolledWindowSetPolicy sw PolicyAutomatic PolicyAutomatic
                     containerAdd widget sw
+#ifdef MIN_VERSION_gtk3
+                    scrolledWindowSetMinContentHeight sw (snd minSize)
+#endif
                     treeSelectionUnselectAll sel
                     let mbInd = elemIndex obj list
                     case mbInd of
@@ -614,7 +671,7 @@ staticListEditor list showF parameters notifier = do
 --
 -- | Editor for the selection of a file path in the form of a text entry and a button,
 -- | which opens a gtk file chooser
-fileEditor :: Maybe FilePath -> FileChooserAction -> String -> Editor FilePath
+fileEditor :: Maybe FilePath -> FileChooserAction -> Text -> Editor FilePath
 fileEditor mbFilePath action buttonName parameters notifier = do
     coreRef <- newIORef Nothing
     mkEditor
@@ -623,11 +680,11 @@ fileEditor mbFilePath action buttonName parameters notifier = do
             case core of
                 Nothing  -> do
                     button <- buttonNewWithLabel buttonName
-                    widgetSetName button $ getParameter paraName parameters ++ "-button"
+                    widgetSetName button $ getParameter paraName parameters <> "-button"
                     mapM_ (activateEvent (castToWidget button) notifier Nothing)
                         (Clicked:genericGUIEvents)
                     entry   <-  entryNew
-                    widgetSetName entry $ getParameter paraName parameters ++ "-entry"
+                    widgetSetName entry $ getParameter paraName parameters <> "-entry"
                     -- set entry [ entryEditable := False ]
                     mapM_ (activateEvent (castToWidget entry) notifier Nothing) genericGUIEvents
                     registerEvent notifier Clicked (buttonHandler entry)
@@ -642,23 +699,23 @@ fileEditor mbFilePath action buttonName parameters notifier = do
                     boxPackStart box entry PackGrow 0
                     boxPackEnd box button PackNatural 0
                     containerAdd widget box
-                    entrySetText entry filePath
+                    entrySetText entry (T.pack filePath)
                     writeIORef coreRef (Just entry)
-                Just entry -> entrySetText entry filePath)
+                Just entry -> entrySetText entry (T.pack filePath))
         (do core <- readIORef coreRef
             case core of
                 Nothing -> return Nothing
                 Just entry -> do
                     str <- entryGetText entry
-                    return (Just str))
+                    return (Just (T.unpack str)))
         parameters
         notifier
     where
     buttonHandler entry e =  do
         mbFileName <- do
             dialog <- fileChooserDialogNew
-                            (Just "Select File")
-                            Nothing
+                        (Just ("Select File"::Text))
+                        Nothing
                         action
                         [("gtk-cancel"
                         ,ResponseCancel)
@@ -684,10 +741,9 @@ fileEditor mbFilePath action buttonName parameters notifier = do
 --                let relative = case mbFilePath of
 --                                Nothing -> fn
 --                                Just rel -> makeRelative rel fn
-                entrySetText entry fn
+                entrySetText entry (T.pack fn)
                 triggerEvent notifier (GUIEvent {
                     selector = MayHaveChanged,
-                    gtkEvent = Gtk.Event True,
                     eventText = "",
                     gtkReturn = True})
                 return (e{gtkReturn=True})
@@ -695,7 +751,7 @@ fileEditor mbFilePath action buttonName parameters notifier = do
 --
 -- | Editor for a font selection
 --
-fontEditor :: Editor (Maybe String)
+fontEditor :: Editor (Maybe Text)
 fontEditor parameters notifier = do
     coreRef <- newIORef Nothing
     mkEditor
@@ -708,9 +764,7 @@ fontEditor parameters notifier = do
                     mapM_ (activateEvent (castToWidget fs) notifier Nothing) (Clicked: genericGUIEvents)
                     activateEvent (castToWidget fs) notifier
                         (Just (\ w h -> do
-                            res     <-  onFontSet (castToFontButton w)  (do
-                                h (Gtk.Event True)
-                                return ())
+                            res     <-  onFontSet (castToFontButton w) (h >> return ())
                             return (unsafeCoerce res))) MayHaveChanged
                     containerAdd widget fs
                     case mbValue of
@@ -747,9 +801,7 @@ colorEditor parameters notifier = do
                     mapM_ (activateEvent (castToWidget cs) notifier Nothing) (Clicked: genericGUIEvents)
                     activateEvent (castToWidget cs) notifier
                         (Just (\ w h -> do
-                            res     <-  onColorSet (castToColorButton w)  (do
-                                h (Gtk.Event True)
-                                return ())
+                            res     <-  onColorSet (castToColorButton w) (h >> return ())
                             return (unsafeCoerce res))) MayHaveChanged
                     containerAdd widget cs
                     colorButtonSetColor cs c
@@ -768,7 +820,7 @@ colorEditor parameters notifier = do
 -- | An editor, which opens another editor
 --   You have to inject a value before the button can be clicked.
 --
-otherEditor :: (alpha  -> String -> IO (Maybe alpha)) -> Editor alpha
+otherEditor :: (alpha  -> Text -> IO (Maybe alpha)) -> Editor alpha
 otherEditor func parameters notifier = do
     coreRef <- newIORef Nothing
     mkEditor
