@@ -1,11 +1,13 @@
-{-# OPTIONS_GHC
-    -XFunctionalDependencies
-    -XNoMonomorphismRestriction
-    -XFlexibleInstances
-    -XMultiParamTypeClasses
-    -XUndecidableInstances
-    -XDeriveDataTypeable
-    -XBangPatterns #-}
+{-# LANGUAGE CPP #-}
+{-# LANGUAGE FunctionalDependencies #-}
+{-# LANGUAGE NoMonomorphismRestriction #-}
+{-# LANGUAGE FlexibleInstances #-}
+{-# LANGUAGE MultiParamTypeClasses #-}
+{-# LANGUAGE UndecidableInstances #-}
+{-# LANGUAGE DeriveDataTypeable #-}
+{-# LANGUAGE BangPatterns #-}
+{-# LANGUAGE ScopedTypeVariables #-}
+{-# LANGUAGE OverloadedStrings #-}
 -----------------------------------------------------------------------------
 --
 -- Module      :  IDE.Core.ViewFrame
@@ -71,6 +73,8 @@ module Graphics.UI.Frame.ViewFrame (
 ,   getUiManager
 ,   getWindows
 ,   getMainWindow
+,   getActiveWindow
+,   getActiveScreen
 ,   getLayout
 ,   getPanesSt
 ,   getPaneMapSt
@@ -90,32 +94,66 @@ module Graphics.UI.Frame.ViewFrame (
 ,   initGtkRc
 ) where
 
-import Graphics.UI.Gtk hiding (afterToggleOverwrite,onToggleOverwrite)
+import Graphics.UI.Gtk
+       (castToNotebook, uiManagerGetAction, Action, Paned,
+        selectionDataGetText, SelectionDataM, InfoId, Point, DragContext,
+        dragDataReceived, dragDestSetTargetList, dragDestSet,
+        notebookSetPopup, notebookSetScrollable, notebookNew,
+        windowPresent, widgetVisible, notebookRemovePage, Notebook,
+        windowGetScreen, Screen, castToWindow, gTypeWindow,
+        widgetGetToplevel, deleteEvent, widgetGetAllocation,
+        windowSetDefaultSize, widgetName, windowNew, Widget,
+        dialogResponse, dialogGetActionArea, castToHBox,
+        dialogGetContentArea, windowTitle, windowTransientFor, dialogNew,
+        Window, widgetDestroy, dialogRun, messageDialogNew, castToWidget,
+        switchPage, widgetGrabFocus, castToVBox, boxReorderChild,
+        castToBox, notebookSetMenuLabel, notebookSetTabLabel,
+        notebookInsertPage, panedPack1, castToContainer, containerRemove,
+        notebookPageNum, panedPack2, hPanedNew, castToPaned, vPanedNew,
+        widgetGetParent, notebookGetCurrentPage, notebookSetTabPos,
+        PositionType, notebookSetShowTabs, notebookGetShowTabs,
+        castToLabel, containerGetChildren, castToBin, binGetChild,
+        notebookGetTabLabel, labelSetMarkup, labelSetUseMarkup,
+        buttonActivated, selectionDataSetText, dragDataGet,
+        dragSourceSetTargetList, targetListAddTextTargets, targetListNew,
+        dragSourceSet, boxPackStart, containerAdd, containerSetBorderWidth,
+        widgetSetVAlign, widgetGetStyleContext, stockClose,
+        imageNewFromStock, imageNewFromPixbuf, iconThemeLoadIcon,
+        iconThemeGetDefault, buttonSetAlignment, buttonSetRelief,
+        buttonSetFocusOnClick, buttonNew, hBoxNew,
+        eventBoxSetVisibleWindow, eventBoxNew, miscSetPadding, castToMisc,
+        miscSetAlignment, EventBox, notebookSetCurrentPage, widgetShowAll,
+        notebookInsertPageMenu, widgetGetName, notebookGetNthPage,
+        notebookGetNPages, labelNew, Label, WidgetClass, NotebookClass,
+        widgetSetName, IconSize(..), ResponseId(..), Rectangle(..),
+        DragAction(..), DestDefaults(..), PositionType(..), Packing(..),
+        ReliefStyle(..), IconLookupFlags(..), after, on, ButtonsType(..),
+        MessageType(..), AttrOp(..), get, set)
 import qualified Data.Map as Map
 import Data.List
 import Data.Maybe
 import Data.Unique
 import Data.Typeable
+import Data.Text (Text)
 
 import Graphics.UI.Frame.Panes
 import Graphics.UI.Editor.Parameters
 import System.Glib (GObjectClass(..), isA)
-#if MIN_VERSION_gtk(0,10,5)
 import Graphics.UI.Gtk.Layout.Notebook (gTypeNotebook)
-#else
-import Graphics.UI.Gtk.Types (gTypeNotebook)
-#endif
 import System.CPUTime (getCPUTime)
-#if MIN_VERSION_gtk(0,10,5)
 import Graphics.UI.Gtk.Gdk.EventM (Modifier(..))
+#ifdef MIN_VERSION_gtk3
+import Graphics.UI.Gtk.General.CssProvider (cssProviderNew, cssProviderLoadFromString)
+import Graphics.UI.Gtk.General.StyleContext (styleContextAddProvider)
+import Graphics.UI.Gtk.General.Enums (Align(..))
 #else
-import Graphics.UI.Gtk.Gdk.Enums (Modifier(..))
+import Graphics.UI.Gtk (rcParseString)
 #endif
-import MyMissing
+import MyMissing (forceJust, forceHead)
 import Graphics.UI.Gtk.Gdk.EventM (TimeStamp(..))
 import Graphics.UI.Editor.MakeEditor
     (mkField, FieldDescription(..), buildEditor)
-import Graphics.UI.Editor.Simple (stringEditor, okCancelFields)
+import Graphics.UI.Editor.Simple (stringEditor, textEditor, okCancelFields)
 import Control.Event (registerEvent)
 import Graphics.UI.Editor.Basics
     (eventText, GUIEventSelector(..))
@@ -124,31 +162,38 @@ import Data.Set (Set(..))
 import Graphics.UI.Gtk.Gdk.Events (Event(..))
 import Control.Monad.IO.Class (MonadIO(..))
 import Control.Monad (when, liftM, foldM)
+import Control.Applicative ((<$>))
 import qualified Control.Monad.Reader as Gtk (liftIO)
+import qualified Data.Text as T (pack, stripPrefix, unpack)
+import Data.Monoid ((<>))
 
 --import Debug.Trace (trace)
-trace a b = b
+trace (a::Text) b = b
 
 groupPrefix = "_group_"
 
-withoutGroupPrefix :: String -> String
-withoutGroupPrefix s = case groupPrefix `stripPrefix` s of
+withoutGroupPrefix :: Text -> Text
+withoutGroupPrefix s = case groupPrefix `T.stripPrefix` s of
                             Nothing -> s
                             Just s' -> s'
 
 
 initGtkRc :: IO ()
-#if MIN_VERSION_gtk(0,11,0)
+#ifdef MIN_VERSION_gtk3
+initGtkRc = return ()
+#else
 initGtkRc = rcParseString ("style \"leksah-close-button-style\"\n" ++
     "{\n" ++
+    "  GtkButton::default-border = 0\n" ++
+    "  GtkButton::default-outside-border = 0\n" ++
+    "  GtkButton::inner-border = 0\n" ++
     "  GtkWidget::focus-padding = 0\n" ++
     "  GtkWidget::focus-line-width = 0\n" ++
     "  xthickness = 0\n" ++
     "  ythickness = 0\n" ++
+    "  padding = 0\n" ++
     "}\n" ++
     "widget \"*.leksah-close-button\" style \"leksah-close-button-style\"")
-#else
-initGtkRc = return ()
 #endif
 
 removePaneAdmin :: RecoverablePane alpha beta delta =>  alpha -> delta ()
@@ -175,7 +220,7 @@ addPaneAdmin pane conn pp = do
             setPanesSt (Map.insert (paneName pane) (PaneC pane) panes')
             return True
         else do
-            trace  ("ViewFrame>addPaneAdmin:pane with this name already exist" ++ paneName pane) $
+            trace ("ViewFrame>addPaneAdmin:pane with this name already exist" <> paneName pane) $
                 return False
 
 getPanePrim ::  RecoverablePane alpha beta delta => delta (Maybe alpha)
@@ -195,12 +240,12 @@ getPanes = do
 notebookInsertOrdered :: PaneMonad alpha => (NotebookClass self, WidgetClass child)		
     => self	
     -> child	-- child - the Widget to use as the contents of the page.
-    -> String
-    -> Maybe Label	-- the label for the page as String or Label
+    -> Text
+    -> Maybe Label	-- the label for the page as Text or Label
     -> Bool
     -> alpha ()
 notebookInsertOrdered nb widget labelStr mbLabel isGroup = do
-    label	    <-  case mbLabel of
+    label       <-  case mbLabel of
                         Nothing  -> liftIO $ labelNew (Just labelStr)
                         Just l  -> return l
     menuLabel   <-  liftIO $ labelNew (Just labelStr)
@@ -219,7 +264,7 @@ notebookInsertOrdered nb widget labelStr mbLabel isGroup = do
         notebookSetCurrentPage nb realPos
 
 -- | Returns a label box
-mkLabelBox :: PaneMonad alpha => Label -> String -> alpha EventBox
+mkLabelBox :: PaneMonad alpha => Label -> Text -> alpha EventBox
 mkLabelBox lbl paneName = do
     (tb,lb) <- liftIO $ do
         miscSetAlignment (castToMisc lbl) 0.0 0.0
@@ -230,26 +275,44 @@ mkLabelBox lbl paneName = do
         innerBox  <- hBoxNew False 0
 
         tabButton <- buttonNew
-        widgetSetName tabButton "leksah-close-button"
+        widgetSetName tabButton ("leksah-close-button"::Text)
         buttonSetFocusOnClick tabButton False
         buttonSetRelief tabButton ReliefNone
         buttonSetAlignment tabButton (0.0,0.0)
 
-        image     <- imageNewFromStock stockClose IconSizeMenu
-        mbPB <- widgetRenderIcon tabButton stockClose IconSizeMenu ""
-        (height,width)   <-  case mbPB of
-                                Nothing -> return (14,14)
-                                Just pb -> do
-                                h <- pixbufGetHeight pb
-                                w <- pixbufGetWidth pb
-                                return (h,w)
-        on tabButton styleSet (\style -> do
-            widgetSetSizeRequest tabButton (height + 2) (width + 2))
+        iconTheme <- iconThemeGetDefault
+        mbIcon <- iconThemeLoadIcon iconTheme ("window-close"::Text) 10 IconLookupUseBuiltin
+        image <- case mbIcon of
+                    Just i  -> imageNewFromPixbuf i
+                    Nothing -> imageNewFromStock stockClose IconSizeMenu
+
+#ifdef MIN_VERSION_gtk3
+        provider <- cssProviderNew
+        cssProviderLoadFromString provider (
+            ".button {\n" <>
+            "-GtkButton-default-border : 0px;\n" <>
+            "-GtkButton-default-outside-border : 0px;\n" <>
+            "-GtkButton-inner-border: 0px;\n" <>
+            "-GtkWidget-focus-line-width : 0px;\n" <>
+            "-GtkWidget-focus-padding : 0px;\n" <>
+            "padding: 0px;\n" <>
+            "border-width: 0px;\n" <>
+            "}\n" <>
+            "GtkImage {\n" <>
+            "padding: 0px;\n" <>
+            "}\n" :: Text)
+        context <- widgetGetStyleContext tabButton
+        styleContextAddProvider context provider 600
+        context <- widgetGetStyleContext image
+        styleContextAddProvider context provider 600
+        widgetSetVAlign tabButton AlignCenter
+        widgetSetVAlign lbl AlignCenter
+#endif
         containerSetBorderWidth tabButton 0
         containerAdd tabButton image
 
-        boxPackStart innerBox lbl PackNatural 0
-        boxPackEnd innerBox tabButton PackNatural 0
+        boxPackStart innerBox tabButton PackNatural 0
+        boxPackStart innerBox lbl       PackGrow 0
 
         containerAdd labelBox innerBox
         dragSourceSet labelBox [Button1] [ActionCopy,ActionMove]
@@ -261,12 +324,12 @@ mkLabelBox lbl paneName = do
             return ())
         return (tabButton,labelBox)
     cl <- runInIO closeHandler
-    liftIO $ onClicked tb (cl ())
+    liftIO $ on tb buttonActivated (cl ())
 
     return lb
     where
         closeHandler :: PaneMonad alpha => () -> alpha ()
-        closeHandler _ =    case groupPrefix `stripPrefix` paneName of
+        closeHandler _ =    case groupPrefix `T.stripPrefix` paneName of
                                 Just group  -> do
                                     closeGroup group
                                 Nothing -> do
@@ -274,12 +337,12 @@ mkLabelBox lbl paneName = do
                                     closePane pane
                                     return ()
 
-groupLabel :: PaneMonad beta => String -> beta EventBox
+groupLabel :: PaneMonad beta => Text -> beta EventBox
 groupLabel group = do
-    label <- liftIO $ labelNew Nothing
+    label <- liftIO $ labelNew (Nothing::Maybe Text)
     liftIO $ labelSetUseMarkup label True
-    liftIO $ labelSetMarkup label ("<b>" ++ group ++ "</b>")
-    labelBox <- mkLabelBox label (groupPrefix ++ group)
+    liftIO $ labelSetMarkup label ("<b>" <> group <> "</b>")
+    labelBox <- mkLabelBox label (groupPrefix <> group)
     liftIO $ widgetShowAll labelBox
     return labelBox
 
@@ -295,16 +358,16 @@ markLabel nb topWidget modified = do
                 Nothing -> return ()
                 Just container -> do
                     children <- containerGetChildren container
-                    let label = castToLabel $ forceHead children "ViewFrame>>markLabel: empty children"
-                    text <- widgetGetName topWidget
+                    let label = castToLabel $ forceHead (tail children) "ViewFrame>>markLabel: empty children"
+                    (text :: Text) <- widgetGetName topWidget
                     labelSetUseMarkup (castToLabel label) True
                     labelSetMarkup (castToLabel label)
                         (if modified
-                              then "<span foreground=\"red\">" ++ text ++ "</span>"
+                              then "<span foreground=\"red\">" <> text <> "</span>"
                           else text)
 
 -- | Constructs a unique pane name, which is an index and a string
-figureOutPaneName :: PaneMonad alpha => String -> Int -> alpha (Int,String)
+figureOutPaneName :: PaneMonad alpha => Text -> Int -> alpha (Int,Text)
 figureOutPaneName bn ind = do
     bufs <- getPanesSt
     let ind = foldr (\(PaneC buf) ind ->
@@ -314,14 +377,14 @@ figureOutPaneName bn ind = do
                 0 (Map.elems bufs)
     if ind == 0
         then return (0,bn)
-        else return (ind,bn ++ "(" ++ show ind ++ ")")
+        else return (ind,bn <> "(" <> T.pack (show ind) <> ")")
 
 paneFromName :: PaneMonad alpha => PaneName -> alpha (IDEPane alpha)
 paneFromName pn = do
     mbPane <- mbPaneFromName pn
     case mbPane of
         Just p -> return p
-        Nothing -> error $ "ViewFrame>>paneFromName:Can't find pane from unique name " ++ pn
+        Nothing -> error $ "ViewFrame>>paneFromName:Can't find pane from unique name " ++ T.unpack pn
 
 mbPaneFromName :: PaneMonad alpha => PaneName -> alpha (Maybe (IDEPane alpha))
 mbPaneFromName pn = do
@@ -334,15 +397,15 @@ guiPropertiesFromName pn = do
     paneMap <- getPaneMapSt
     case Map.lookup pn paneMap of
             Just it -> return it
-            otherwise  -> error $"Cant't find guiProperties from unique name " ++ pn
+            otherwise  -> error $"Cant't find guiProperties from unique name " ++ T.unpack pn
 
 posTypeToPaneDirection PosLeft      =   LeftP
-posTypeToPaneDirection PosRight     =   RightP	
+posTypeToPaneDirection PosRight     =   RightP
 posTypeToPaneDirection PosTop       =   TopP
-posTypeToPaneDirection PosBottom    =   BottomP	
+posTypeToPaneDirection PosBottom    =   BottomP
 
 paneDirectionToPosType LeftP        =   PosLeft
-paneDirectionToPosType RightP       =   PosRight   	
+paneDirectionToPosType RightP       =   PosRight
 paneDirectionToPosType TopP         =   PosTop
 paneDirectionToPosType BottomP      =   PosBottom
 
@@ -427,15 +490,15 @@ viewSplit' panePath dir = do
                                                                  return (castToPaned h)
                                               Vertical    -> do  v <- hPanedNew
                                                                  return (castToPaned v)
-                                rName <- widgetGetName activeNotebook
+                                rName::Text <- widgetGetName activeNotebook
                                 widgetSetName newpane rName
-                                widgetSetName nb altname
+                                widgetSetName nb (altname :: Text)
                                 panedPack2 newpane nb True True
                                 nbIndex <- if parent `isA` gTypeNotebook
                                             then notebookPageNum ((castToNotebook' "viewSplit'1") parent) activeNotebook
                                             else trace ("ViewFrame>>viewSplit': parent not a notebook: ") return Nothing
                                 containerRemove (castToContainer parent) activeNotebook
-                                widgetSetName activeNotebook name
+                                widgetSetName activeNotebook (name :: Text)
                                 panedPack1 newpane activeNotebook True True
                                 return (newpane,nbIndex)
                             case (reverse panePath, nbi) of
@@ -463,7 +526,7 @@ viewSplit' panePath dir = do
                                     _      -> trace ("ViewFrame>>viewSplit': parent not a notebook2: ")return ()
                                 return (nb,paneDir)
                         handleFunc <-  runInIO (handleNotebookSwitch nb)
-                        liftIO $ afterSwitchPage nb handleFunc
+                        liftIO $ after nb switchPage handleFunc
                         return (Just (paneDir,dir))
             case mbPD of
               Just (paneDir,pdir) -> do
@@ -524,7 +587,7 @@ viewCollapse' panePath = trace "viewCollapse' called" $ do
                                                         $Map.toList paneMap
                             panesToMove       <- mapM paneFromName paneNamesToMove
                             mapM_ (\(PaneC p) -> move panePath p) panesToMove
-                            let groupNames    =  map (\n -> groupPrefix ++ n) $
+                            let groupNames    =  map (\n -> groupPrefix <> n) $
                                                         getGroupsFrom otherSidePath layout1
                             mapM_ (\n -> move' (n,activeNotebook)) groupNames
                             -- 2. Remove unused notebook from admin
@@ -565,7 +628,7 @@ viewCollapse' panePath = trace "viewCollapse' called" $ do
                                                 else liftIO $ do
                                                     boxPackStart (castToVBox grandparent) activeNotebook PackGrow 0
                                                     boxReorderChild (castToVBox grandparent) activeNotebook 2
-                                                    widgetSetName activeNotebook "root"
+                                                    widgetSetName activeNotebook ("root" :: Text)
                             -- 4. Change panePathFromNotebook
                             adjustNotebooks panePath newPanePath
                             -- 5. Change paneMap
@@ -573,7 +636,7 @@ viewCollapse' panePath = trace "viewCollapse' called" $ do
                             -- 6. Change layout
                             adjustLayoutForCollapse panePath
 
-getGroupsFrom :: PanePath -> PaneLayout -> [String]
+getGroupsFrom :: PanePath -> PaneLayout -> [Text]
 getGroupsFrom path layout =
     case layoutFromPath path layout of
         t@(TerminalP _ _ _ _ _)   -> Map.keys (paneGroups t)
@@ -591,14 +654,14 @@ viewNewGroup = do
             if groupName `Set.member` allGroupNames layout
                 then liftIO $ do
                     md <- messageDialogNew (Just mainWindow) [] MessageWarning ButtonsClose
-                        ("Group name not unique " ++ groupName)
+                        ("Group name not unique " <> groupName)
                     dialogRun md
                     widgetDestroy md
                     return ()
                 else viewNest groupName
         Nothing -> return ()
 
-newGroupOrBringToFront :: PaneMonad alpha => String -> PanePath -> alpha (Maybe PanePath,Bool)
+newGroupOrBringToFront :: PaneMonad alpha => Text -> PanePath -> alpha (Maybe PanePath,Bool)
 newGroupOrBringToFront groupName pp = do
     layout <- getLayoutSt
     if groupName `Set.member` allGroupNames layout
@@ -609,7 +672,7 @@ newGroupOrBringToFront groupName pp = do
             viewNest' realPath groupName
             return (Just (realPath ++ [GroupP groupName]),True)
 
-bringGroupToFront :: PaneMonad alpha => String -> alpha (Maybe PanePath)
+bringGroupToFront :: PaneMonad alpha => Text -> alpha (Maybe PanePath)
 bringGroupToFront groupName = do
     layout <- getLayoutSt
     case findGroupPath groupName layout   of
@@ -622,13 +685,17 @@ bringGroupToFront groupName = do
 
 --  Yet another stupid little dialog
 
-groupNameDialog :: Window -> IO (Maybe String)
+groupNameDialog :: Window -> IO (Maybe Text)
 groupNameDialog parent =  liftIO $ do
     dia                        <-   dialogNew
-    windowSetTransientFor dia parent
-    windowSetTitle dia "Enter group name"
-    upper                      <-   dialogGetUpper dia
-    lower                      <-   dialogGetActionArea dia
+    set dia [ windowTransientFor := parent
+            , windowTitle        := ("Enter group name" :: Text) ]
+#ifdef MIN_VERSION_gtk3
+    upper                      <-   castToVBox <$> dialogGetContentArea dia
+#else
+    upper                      <-   castToVBox <$> dialogGetUpper dia
+#endif
+    lower                      <-   castToHBox <$> dialogGetActionArea dia
     (widget,inj,ext,_)         <-   buildEditor moduleFields ""
     (widget2,_,_,notifier)     <-   buildEditor okCancelFields ()
     registerEvent notifier ButtonPressed (\e -> do
@@ -639,23 +706,23 @@ groupNameDialog parent =  liftIO $ do
     boxPackStart upper widget PackGrow 7
     boxPackStart lower widget2 PackNatural 7
     widgetShowAll dia
-    resp <- dialogRun dia
-    value                      <- ext ("")
+    resp  <- dialogRun dia
+    value <- ext ("")
     widgetDestroy dia
     case resp of
-        ResponseOk | value /= Just ""  -> return value
+        ResponseOk | value /= Just "" -> return value
         _                             -> return Nothing
     where
-        moduleFields :: FieldDescription String
+        moduleFields :: FieldDescription Text
         moduleFields = VFD emptyParams [
                 mkField
                     (paraName <<<- ParaName ("New group ")
                             $ emptyParams)
                     id
                     (\ a b -> a)
-            (stringEditor (const True) True)]
+            (textEditor (const True) True)]
 
-viewNest :: PaneMonad alpha => String -> alpha ()
+viewNest :: PaneMonad alpha => Text -> alpha ()
 viewNest group = do
     mbPanePath        <- getActivePanePath
     case mbPanePath of
@@ -663,7 +730,7 @@ viewNest group = do
         Just panePath -> do
             viewNest' panePath group
 
-viewNest' :: PaneMonad alpha => PanePath -> String -> alpha ()
+viewNest' :: PaneMonad alpha => PanePath -> Text -> alpha ()
 viewNest' panePath group = do
     activeNotebook  <- (getNotebook' "viewNest' 1") panePath
     mbParent  <- liftIO $ widgetGetParent activeNotebook
@@ -675,22 +742,22 @@ viewNest' panePath group = do
             case paneLayout of
                 (TerminalP {}) -> do
                     nb <- newNotebook (panePath ++ [GroupP group])
-                    liftIO $ widgetSetName nb (groupPrefix ++ group)
+                    liftIO $ widgetSetName nb (groupPrefix <> group)
                     notebookInsertOrdered activeNotebook nb group Nothing True
                     liftIO $ widgetShowAll nb
                         --widgetGrabFocus activeNotebook
                     handleFunc <-  runInIO (handleNotebookSwitch nb)
-                    liftIO $ afterSwitchPage nb handleFunc
+                    liftIO $ after nb switchPage handleFunc
                     adjustLayoutForNest group panePath
                 _ -> return ()
 
-closeGroup :: PaneMonad alpha => String -> alpha ()
+closeGroup :: PaneMonad alpha => Text -> alpha ()
 closeGroup groupName = do
     layout <- getLayout
     let mbPath = findGroupPath groupName layout
     mainWindow <- getMainWindow
     case mbPath of
-        Nothing -> trace ("ViewFrame>>closeGroup: Group path not found: " ++ groupName) return ()
+        Nothing -> trace ("ViewFrame>>closeGroup: Group path not found: " <> groupName) return ()
         Just path -> do
             panesMap <- getPaneMapSt
             let nameAndpathList  = filter (\(a,pp) -> path `isPrefixOf` pp)
@@ -698,7 +765,7 @@ closeGroup groupName = do
             continue <- case nameAndpathList of
                             (_:_) -> liftIO $ do
                                 md <- messageDialogNew (Just mainWindow) [] MessageQuestion ButtonsYesNo
-                                    ("Group " ++ groupName ++ " not empty. Close with all contents?")
+                                    ("Group " <> groupName <> " not empty. Close with all contents?")
                                 rid <- dialogRun md
                                 widgetDestroy md
                                 case rid of
@@ -725,9 +792,9 @@ viewDetach = do
     case mbPanePath of
         Nothing -> return Nothing
         Just panePath -> do
-            viewDetach' panePath id
+            viewDetach' panePath (T.pack id)
 
-viewDetach' :: PaneMonad alpha => PanePath -> String -> alpha (Maybe (Window,Widget))
+viewDetach' :: PaneMonad alpha => PanePath -> Text -> alpha (Maybe (Window,Widget))
 viewDetach' panePath id = do
     activeNotebook  <- (getNotebook' "viewDetach'") panePath
     mbParent  <- liftIO $ widgetGetParent activeNotebook
@@ -740,20 +807,20 @@ viewDetach' panePath id = do
                 (TerminalP{detachedSize = size}) -> do
                     window <- liftIO $ do
                         window <- windowNew
-                        windowSetTitle window "Leksah detached window"
-                        widgetSetName window id
+                        set window [ windowTitle := ("Leksah detached window" :: Text)
+                                   , widgetName  := Just id ]
                         case size of
                             Just (width, height) -> do
                                 windowSetDefaultSize window width height
                             Nothing -> do
-                                (curWidth, curHeight) <- widgetGetSize activeNotebook
+                                (Rectangle _ _ curWidth curHeight) <- widgetGetAllocation activeNotebook
                                 windowSetDefaultSize window curWidth curHeight
                         containerRemove (castToContainer parent) activeNotebook
                         containerAdd window activeNotebook
                         widgetShowAll window
                         return window
                     handleFunc <-  runInIO (handleReattach id window)
-                    liftIO $ window `onDelete` handleFunc
+                    liftIO . on window deleteEvent . liftIO $ handleFunc ()
                     windows <- getWindowsSt
                     setWindowsSt $ windows ++ [window]
                     adjustLayoutForDetach id panePath
@@ -762,11 +829,11 @@ viewDetach' panePath id = do
 
 
 
-handleReattach :: PaneMonad alpha => String -> Window -> Event -> alpha Bool
+handleReattach :: PaneMonad alpha => Text -> Window -> () -> alpha Bool
 handleReattach windowId window _ = do
     layout <- getLayout
     case findDetachedPath windowId layout of
-        Nothing -> trace ("ViewFrame>>handleReattach: panePath for id not found: " ++ windowId)
+        Nothing -> trace ("ViewFrame>>handleReattach: panePath for id not found: " <> windowId)
                 $ do
             windows <- getWindowsSt
             setWindowsSt $ delete window windows
@@ -787,8 +854,26 @@ handleReattach windowId window _ = do
             return False -- "now destroy the window"
 
 
+getActiveWindow :: PaneMonad alpha => alpha (Maybe Window)
+getActiveWindow = do
+    mbPanePath <- getActivePanePath
+    case mbPanePath of
+        Nothing -> return Nothing
+        Just panePath -> do
+            activeNotebook  <- (getNotebook' "getActiveWindow") panePath
+            top <- liftIO $ widgetGetToplevel activeNotebook
+            if (top `isA` gTypeWindow)
+                then return . Just $ castToWindow top
+                else return Nothing
 
-groupMenuLabel :: PaneMonad beta => String -> beta (Maybe Label)
+getActiveScreen :: PaneMonad alpha => alpha (Maybe Screen)
+getActiveScreen = do
+    mbWindow <- getActiveWindow
+    case mbWindow of
+        Nothing -> return Nothing
+        Just window -> liftIO $ Just <$> windowGetScreen window
+
+groupMenuLabel :: PaneMonad beta => Text -> beta (Maybe Label)
 groupMenuLabel group = liftM Just (liftIO $ labelNew (Just group))
 
 handleNotebookSwitch :: PaneMonad beta => Notebook -> Int -> beta ()
@@ -803,7 +888,7 @@ handleNotebookSwitch nb index = do
                 Nothing         ->  return ()
                 Just (PaneC p)  ->  makeActive p
     where
-        findPaneFor :: PaneMonad beta => String -> beta (Maybe (IDEPane beta))
+        findPaneFor :: PaneMonad beta => Text -> beta (Maybe (IDEPane beta))
         findPaneFor n1   =   do
             panes'      <-  getPanesSt
             foldM (\r (PaneC p) -> do
@@ -872,10 +957,10 @@ move' (paneName,toNB) = do
     panes           <-  getPanesSt
     layout          <-  getLayout
     frameState      <-  getFrameState
-    case groupPrefix `stripPrefix` paneName of
+    case groupPrefix `T.stripPrefix` paneName of
         Just group  -> do
             case findGroupPath group layout of
-                Nothing -> trace ("ViewFrame>>move': group not found: " ++ group) return ()
+                Nothing -> trace ("ViewFrame>>move': group not found: " <> group) return ()
                 Just fromPath -> do
                     groupNBOrPaned <- getNotebookOrPaned fromPath castToWidget
                     fromNB  <- (getNotebook' "move'") (init fromPath)
@@ -898,13 +983,13 @@ move' (paneName,toNB) = do
                                         return ()
         Nothing     ->
             case paneName `Map.lookup` panes of
-                Nothing -> trace ("ViewFrame>>move': pane not found: " ++ paneName) return ()
+                Nothing -> trace ("ViewFrame>>move': pane not found: " <> paneName) return ()
                 Just (PaneC pane) -> do
                     case toNB `Map.lookup` (panePathFromNB frameState) of
                         Nothing -> trace "ViewFrame>>move': panepath for Notebook not found2" return ()
                         Just toPath ->
                             case paneName `Map.lookup`paneMap of
-                                Nothing -> trace ("ViewFrame>>move': pane data not found: " ++ paneName)
+                                Nothing -> trace ("ViewFrame>>move': pane data not found: " <> paneName)
                                             return ()
                                 Just (fromPath,_) -> do
                                     let child = getTopWidget pane
@@ -937,6 +1022,9 @@ findAppropriate  (VerticalP l r _) TopP        =   SplitP LeftP   :  findAppropr
 bringPaneToFront :: RecoverablePane alpha beta delta => alpha -> IO ()
 bringPaneToFront pane = do
     let tv = getTopWidget pane
+    w <- widgetGetToplevel tv
+    visible <- w `get` widgetVisible
+    when visible . windowPresent $ castToWindow w
     setCurrentNotebookPages tv
 
 
@@ -976,7 +1064,7 @@ getBestPanePath sp pl = reverse $ getStandard' sp pl []
 --
 -- | Get a standard path.
 --
-getBestPathForId :: PaneMonad alpha => String -> alpha PanePath
+getBestPathForId :: PaneMonad alpha => Text -> alpha PanePath
 getBestPathForId  id = do
     p <- panePathForGroup id
     l <- getLayout
@@ -1038,30 +1126,30 @@ terminalsWithPanePath pl = map (\ (pp,l) -> (reverse pp,l)) $ terminalsWithPP []
                                                         ++ terminalsWithPP (SplitP BottomP : pp) b
         terminalsFromGroup pp (name,layout)        =  terminalsWithPP (GroupP name : pp) layout
 
-findGroupPath :: String -> PaneLayout -> Maybe PanePath
+findGroupPath :: Text -> PaneLayout -> Maybe PanePath
 findGroupPath group layout =
     let terminalPairs = terminalsWithPanePath layout
     in case (filter filterFunc terminalPairs) of
         [] -> Nothing
         (pp,_) : [] -> Just (pp ++ [GroupP group])
-        _ -> error ("ViewFrame>>group name not unique: " ++ group)
+        _ -> error ("ViewFrame>>group name not unique: " ++ T.unpack group)
     where
         filterFunc (_,(TerminalP groups _ _ _ _)) =  group  `Set.member` Map.keysSet groups
         filterFunc _                              =  error "ViewFrame>>findGroupPath: impossible"
 
-findDetachedPath :: String -> PaneLayout -> Maybe PanePath
+findDetachedPath :: Text -> PaneLayout -> Maybe PanePath
 findDetachedPath id layout =
     let terminalPairs = terminalsWithPanePath layout
     in case (filter filterFunc terminalPairs) of
         [] -> Nothing
         (pp,_) : [] -> Just pp
-        _ -> error ("ViewFrame>>window id not unique: " ++ id)
+        _ -> error ("ViewFrame>>window id not unique: " ++ T.unpack id)
     where
         filterFunc (_,(TerminalP _ _ _ (Just lid) _)) = lid == id
         filterFunc _                                  = False
 
 
-allGroupNames :: PaneLayout -> Set String
+allGroupNames :: PaneLayout -> Set Text
 allGroupNames pl = Set.unions $ map getFunc (terminalsWithPanePath pl)
     where
         getFunc (_,(TerminalP groups _ _ _ _)) =  Map.keysSet groups
@@ -1113,7 +1201,7 @@ layoutsFromPath (SplitP RightP:r) layout@(VerticalP l ri _)   =   layout:layouts
 layoutsFromPath pp l                                      = error
     $"inconsistent layout (layoutsFromPath) " ++ show pp ++ " " ++ show l
 
-getWidgetNameList :: PanePath -> PaneLayout -> [String]
+getWidgetNameList :: PanePath -> PaneLayout -> [Text]
 getWidgetNameList path layout = reverse $ nameList (reverse path) (reverse $ layoutsFromPath path layout)
     where
         nameList [] _ = reverse ["Leksah Main Window","topBox","root"]
@@ -1130,10 +1218,10 @@ getNotebookOrPaned p cf = do
 -- | Get the notebook widget for the given pane path
 --
 getNotebook :: PaneMonad alpha => PanePath -> alpha  Notebook
-getNotebook p = getNotebookOrPaned p (castToNotebook' ("getNotebook " ++ show p))
+getNotebook p = getNotebookOrPaned p (castToNotebook' ("getNotebook " <> T.pack (show p)))
 
-getNotebook' :: PaneMonad alpha => String -> PanePath -> alpha  Notebook
-getNotebook' str p = getNotebookOrPaned p (castToNotebook' ("getNotebook' " ++ str ++ " " ++ show p))
+getNotebook' :: PaneMonad alpha => Text -> PanePath -> alpha  Notebook
+getNotebook' str p = getNotebookOrPaned p (castToNotebook' ("getNotebook' " <> str <> " " <> T.pack (show p)))
 
 
 --
@@ -1179,15 +1267,15 @@ getActiveNotebook = do
 --
 -- | Translates a pane direction to the widget name
 --
-paneDirectionToWidgetName           :: PaneDirection -> String
+paneDirectionToWidgetName           :: PaneDirection -> Text
 paneDirectionToWidgetName TopP      =  "top"
 paneDirectionToWidgetName BottomP   =  "bottom"
 paneDirectionToWidgetName LeftP     =  "left"
 paneDirectionToWidgetName RightP    =  "right"
 
-panePathElementToWidgetName :: PanePathElement -> String
+panePathElementToWidgetName :: PanePathElement -> Text
 panePathElementToWidgetName (SplitP dir)   = paneDirectionToWidgetName dir
-panePathElementToWidgetName (GroupP group) = groupPrefix ++ group
+panePathElementToWidgetName (GroupP group) = groupPrefix <> group
 
 --
 -- | Changes a pane path in the pane map
@@ -1202,7 +1290,7 @@ adjustPanes fromPane toPane  = do
 
 adjustNotebooks :: PaneMonad alpha => PanePath -> PanePath -> alpha ()
 adjustNotebooks fromPane toPane  = do
-    npMap <- trace ("+++ adjustNotebooks from: " ++ show fromPane ++ " to " ++ show toPane)
+    npMap <- trace ("+++ adjustNotebooks from: " <> T.pack (show fromPane) <> " to " <> T.pack (show toPane))
                 getPanePathFromNB
     setPanePathFromNB  (Map.map (\pp ->
         case stripPrefix fromPane pp of
@@ -1227,7 +1315,7 @@ adjustLayoutForSplit  dir path  = do
 --
 -- | Changes the layout for a nest
 --
-adjustLayoutForNest :: PaneMonad alpha => String -> PanePath -> alpha ()
+adjustLayoutForNest :: PaneMonad alpha => Text -> PanePath -> alpha ()
 adjustLayoutForNest group path = do
     layout          <-  getLayoutSt
     let paneLayout  =   layoutFromPath path layout
@@ -1240,7 +1328,7 @@ adjustLayoutForNest group path = do
 --
 -- | Changes the layout for a detach
 --
-adjustLayoutForDetach :: PaneMonad alpha => String -> PanePath -> alpha ()
+adjustLayoutForDetach :: PaneMonad alpha => Text -> PanePath -> alpha ()
 adjustLayoutForDetach id path = do
     layout          <-  getLayoutSt
     let paneLayout  =   layoutFromPath path layout
@@ -1273,7 +1361,7 @@ adjustLayoutForCollapse oldPath  = do
 --
 -- | Changes the layout for a move
 --
-adjustLayoutForGroupMove :: PaneMonad alpha => PanePath -> PanePath -> String -> alpha ()
+adjustLayoutForGroupMove :: PaneMonad alpha => PanePath -> PanePath -> Text -> alpha ()
 adjustLayoutForGroupMove fromPath toPath group = do
     layout <- getLayout
     let layoutToMove = layoutFromPath fromPath layout
@@ -1283,7 +1371,7 @@ adjustLayoutForGroupMove fromPath toPath group = do
 --
 -- | Changes the layout for a remove
 --
-adjustLayoutForGroupRemove :: PaneMonad alpha => PanePath -> String -> alpha ()
+adjustLayoutForGroupRemove :: PaneMonad alpha => PanePath -> Text -> alpha ()
 adjustLayoutForGroupRemove fromPath group = do
     layout <- getLayout
     setLayoutSt (removeGL fromPath layout)
@@ -1335,13 +1423,13 @@ adjustLayout pp layout replace    = adjust' pp layout
 --
 -- | Get the widget from a list of strings
 --
-widgetFromPath :: Widget -> [String] -> IO (Widget)
+widgetFromPath :: Widget -> [Text] -> IO (Widget)
 widgetFromPath w [] = return w
 widgetFromPath w path = do
     children    <- containerGetChildren (castToContainer w)
     chooseWidgetFromPath children path
 
-chooseWidgetFromPath :: [Widget] -> [String] -> IO (Widget)
+chooseWidgetFromPath :: [Widget] -> [Text] -> IO (Widget)
 chooseWidgetFromPath _ [] = error $"Cant't find widget (empty path)"
 chooseWidgetFromPath widgets (h:t) = do
     names       <- mapM widgetGetName widgets
@@ -1350,25 +1438,25 @@ chooseWidgetFromPath widgets (h:t) = do
         Nothing     -> error $"Cant't find widget path " ++ show (h:t) ++ " found only " ++ show names
         Just ind    -> widgetFromPath (widgets !! ind) t
 
-widgetGet :: PaneMonad alpha => [String] -> (Widget -> b) -> alpha  (b)
+widgetGet :: PaneMonad alpha => [Text] -> (Widget -> b) -> alpha  (b)
 widgetGet strL cf = do
     windows <- getWindowsSt
     r <- liftIO $chooseWidgetFromPath (map castToWidget windows) strL
     return (cf r)
 
-widgetGetRel :: Widget -> [String] -> (Widget -> b) -> IO (b)
+widgetGetRel :: Widget -> [Text] -> (Widget -> b) -> IO (b)
 widgetGetRel w sl cf = do
     r <- widgetFromPath w sl
     return (cf r)
 
-getUIAction :: PaneMonad alpha => String -> (Action -> a) -> alpha (a)
+getUIAction :: PaneMonad alpha => Text -> (Action -> a) -> alpha (a)
 getUIAction str f = do
     uiManager <- getUiManagerSt
     liftIO $ do
         findAction <- uiManagerGetAction uiManager str
         case findAction of
             Just act -> return (f act)
-            Nothing  -> error $"getUIAction can't find action " ++ str
+            Nothing  -> error $"getUIAction can't find action " ++ T.unpack str
 
 getThis :: PaneMonad delta =>  (FrameState delta -> alpha) -> delta alpha
 getThis sel = do
@@ -1377,7 +1465,7 @@ getThis sel = do
 setThis :: PaneMonad delta =>  (FrameState delta -> alpha -> FrameState delta) -> alpha -> delta ()
 setThis sel value = do
     st <- getFrameState
-    trace ("!!! setFrameState " ++ show (sel st value)) $ setFrameState (sel st value)
+    trace ("!!! setFrameState " <> T.pack (show $ sel st value)) $ setFrameState (sel st value)
 
 getWindowsSt    = getThis windows
 setWindowsSt    = setThis (\st value -> st{windows = value})
@@ -1400,8 +1488,8 @@ getWindows      = getWindowsSt
 getMainWindow   = liftM head getWindows
 getLayout       = getLayoutSt
 
-castToNotebook' :: GObjectClass obj => String -> obj -> Notebook
+castToNotebook' :: GObjectClass obj => Text -> obj -> Notebook
 castToNotebook' str obj = if obj `isA` gTypeNotebook
                             then castToNotebook obj
-                            else error ("Not a notebook " ++ str)
+                            else error . T.unpack $ "Not a notebook " <> str
 
